@@ -46,12 +46,12 @@
 #ifndef	CCXX_PERSIST_H_
 #define	CCXX_PERSIST_H_
 
-#ifndef CCXX_CONFIG_H_
-#include <cc++/config.h>
+#ifndef CCXX_MISSING_H_
+#include <cc++/missing.h>
 #endif
 
-#ifndef	CCXX_STRCHAR_H_
-#include <cc++/persist.h>
+#ifndef	CCXX_STRING_H_
+#include <cc++/string.h>
 #endif
 
 #ifdef HAVE_ZLIB_H
@@ -70,30 +70,35 @@
 
 #ifdef CCXX_NAMESPACES
 namespace ost {
+#define NS_PREFIX ost::
+#else
+#define	NS_PREFIX
 #endif
 
+#ifdef	CCXX_EXCEPTIONS
 #ifdef	COMMON_STD_EXCEPTION
 
-class CCXX_CLASS_EXPORT PersistException
+class __EXPORT PersistException : public Exception
 {
 public:
-	PersistException(const std::string &what_arg) throw();
-	virtual ~PersistException() throw();
-	const std::string &getString() const;
+	PersistException(const String &what) : Exception(what) {};
 };
 
 #else
 
-class PersistException
+class __EXPORT PersistException
 {
 public:
-	CCXX_MEMBER_EXPORT(CCXX_EMPTY) PersistException(const std::string& reason);
-	CCXX_MEMBER_EXPORT(const std::string&) getString() const;
-	CCXX_MEMBER_EXPORT(virtual) ~PersistException();
+	PersistException(const String& reason);
+	inline const String& getString() const
+		{return Exception::getString();};
+
+	virtual ~PersistException() {} throw();
 protected:
-	std::string _what;
+	String _what;
 };
 
+#endif
 #endif
 
 // This typedef allows us to declare NewBaseObjectFunction now
@@ -107,7 +112,7 @@ typedef class BaseObject* (*NewBaseObjectFunction) (void);
  * @author Daniel Silverstone
  * @short Type manager for persistence engine.
  */
-class TypeManager
+class __EXPORT TypeManager
 {
 public:
 	
@@ -122,27 +127,27 @@ public:
 			: myName(name) { TypeManager::add(name,func); }
 		virtual ~Registration() { TypeManager::remove(myName.c_str()); }
 	private:
-		std::string myName;
+		String myName;
 	};
 	
 	/**
 	 * This adds a new construction function to the type manager
 	 */
-	CCXX_MEMBER_EXPORT(static void) add(const char* name, NewBaseObjectFunction construction);
+	static void add(const char* name, NewBaseObjectFunction construction);
 	
 	/**
 	 * And this one removes a type from the managers lists
 	 */
-	CCXX_MEMBER_EXPORT(static void) remove(const char* name);
+	static void remove(const char* name);
 	
 	/**
 	 * This function creates a new object of the required type and
 	 * returns a pointer to it. NULL is returned if we couldn't find 
 	 * the type
 	 */
-	CCXX_EXPORT(static BaseObject*) createInstanceOf(const char* name);
+	static BaseObject* createInstanceOf(const char* name);
 	
-	typedef std::map<std::string,NewBaseObjectFunction> StringFunctionMap;
+	typedef std::map<String,NewBaseObjectFunction> StringFunctionMap;
 };
 
 
@@ -153,22 +158,22 @@ public:
 
 #define DECLARE_PERSISTENCE(ClassType)					\
   public:								\
-    friend Engine& operator>>(Engine& ar, ClassType *&ob);		\
-    friend Engine& operator<<(Engine& ar, ClassType const &ob);	\
-    friend BaseObject *createNew##ClassType();				\
+    friend NS_PREFIX Engine& operator>>( NS_PREFIX Engine& ar, ClassType *&ob);		\
+    friend NS_PREFIX Engine& operator<<( NS_PREFIX Engine& ar, ClassType const &ob);	\
+    friend NS_PREFIX BaseObject *createNew##ClassType();				\
     virtual const char* getPersistenceID() const;			\
-    static TypeManager::Registration registrationFor##ClassType;
+    static NS_PREFIX TypeManager::Registration registrationFor##ClassType;
 
 #define IMPLEMENT_PERSISTENCE(ClassType, FullyQualifiedName)		      \
-  BaseObject *createNew##ClassType() { return new ClassType; }		      \
+  NS_PREFIX BaseObject *createNew##ClassType() { return new ClassType; }		      \
   const char* ClassType::getPersistenceID() const {return FullyQualifiedName;} \
-  Engine& operator>>(Engine& ar, ClassType &ob)			      \
-    { ar >> (BaseObject &) ob; return ar; }				      \
-  Engine& operator>>(Engine& ar, ClassType *&ob)			      \
-    { ar >> (BaseObject *&) ob; return ar; }				      \
-  Engine& operator<<(Engine& ar, ClassType const &ob)			      \
-    { ar << (BaseObject const *)&ob; return ar; }			      \
-  TypeManager::Registration						      \
+  NS_PREFIX Engine& operator>>(NS_PREFIX Engine& ar, ClassType &ob)			      \
+    { ar >> (NS_PREFIX BaseObject &) ob; return ar; }				      \
+  NS_PREFIX Engine& operator>>(NS_PREFIX Engine& ar, ClassType *&ob)			      \
+    { ar >> (NS_PREFIX BaseObject *&) ob; return ar; }				      \
+  NS_PREFIX Engine& operator<<(NS_PREFIX Engine& ar, ClassType const &ob)			      \
+    { ar << (NS_PREFIX BaseObject const *)&ob; return ar; }			      \
+  NS_PREFIX TypeManager::Registration						      \
         ClassType::registrationFor##ClassType(FullyQualifiedName,	      \
 					      createNew##ClassType);
 
@@ -193,7 +198,7 @@ class Engine;
  * @author Daniel Silverstone
  * @short Base class for classes that will be persistent.
  */
-class BaseObject
+class __EXPORT BaseObject
 {
 public:
 	/**
@@ -201,31 +206,31 @@ public:
 	 * It is called in CreateNewInstance in order to create
 	 * an instance of the class to have Read() called on it.
 	 */
-	CCXX_MEMBER_EXPORT(CCXX_EMPTY) BaseObject();
+	 BaseObject();
 	
 	/**
 	 * Default destructor
 	 */
-	CCXX_MEMBER_EXPORT(virtual) ~BaseObject();
+	 virtual ~BaseObject();
 	
 	/**
 	 * This returns the ID of the persistent object (Its type)
 	 */
-	CCXX_MEMBER_EXPORT(virtual const char*) getPersistenceID() const;
+	 virtual const char* getPersistenceID() const;
 	
 	/**
 	 * This method is used to write to the Persistence::Engine
 	 * It is not equivalent to the << operator as it writes only the data
 	 * and not the object type etc.
 	 */
-	CCXX_MEMBER_EXPORT(virtual bool) write(Engine& archive) const;
+	 virtual bool write(Engine& archive) const;
 
 	/**
 	 * This method is used to read from a Persistence::Engine
 	 * It is not equivalent to the >> operator as it does no 
 	 * typesafety or anything.
 	 */
-	CCXX_MEMBER_EXPORT(virtual bool) read(Engine& archive);
+	 virtual bool read(Engine& archive);
 };
 
 
@@ -240,18 +245,20 @@ public:
  * @short stream serialization of persistent classes.
  */
 
-class Engine
+class __EXPORT Engine
 {
 public:
 	/**
 	 * If problems happen which are fatal - expect one of these thrown 
 	 * at you
 	 */
+#ifdef	CCXX_EXCEPTIONS
 	class Exception : public PersistException
 	{
 	public:
-		CCXX_MEMBER_EXPORT(CCXX_EMPTY) Exception(const std::string &reason);
+		Exception(const String &reason);
 	};
+#endif
 	
 	/**
 	 * These are the modes the Persistence::Engine can work in
@@ -267,13 +274,13 @@ public:
 	 * the given mode. The stream must be initialised properly prior
 	 * to this call or problems will ensue.
 	 */
-	CCXX_MEMBER_EXPORT(CCXX_EMPTY) Engine(std::iostream& stream, EngineMode mode) THROWS (PersistException);
+	Engine(std::iostream& stream, EngineMode mode) THROWS (PersistException);
 	
 	/**
 	 * This Flushes the buffers and closes the Persistence::Engine
 	 * this must happen before the underlying stream is shut down
 	 */
-	CCXX_MEMBER_EXPORT(virtual) ~Engine();
+	virtual ~Engine();
 
 
 	// Write operations
@@ -281,13 +288,13 @@ public:
 	/**
 	 * writes a BaseObject from a reference.
 	 */
-	CCXX_MEMBER_EXPORT(void) write(const BaseObject &object) THROWS (Exception)
-  { write(&object); };
+	void write(const BaseObject &object) THROWS (Exception)
+	{ write(&object); };
 
 	/**
 	 * writes a BaseObject from a pointer.
 	 */
-	CCXX_MEMBER_EXPORT(void) write(const BaseObject *object) THROWS (Exception);
+	void write(const BaseObject *object) THROWS (Exception);
 
 	// writes supported primitive types
   // shortcut, to make the following more readable
@@ -298,15 +305,17 @@ public:
 	void write(uint16 i) THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
 	void write(int32 i)  THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
 	void write(uint32 i) THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
+#ifdef	HAVE_64_BITS
 	void write(int64 i)  THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
 	void write(uint64 i) THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
+#endif
 	void write(float i)  THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
 	void write(double i) THROWS (Exception) { CCXX_ENGINEWRITE_REF(i); }
 #undef CCXX_ENGINEWRITE_REF
-	CCXX_MEMBER_EXPORT(void) write(const std::string& str) THROWS (Exception);
+	void write(const String& str) THROWS (Exception);
 
 	// Every write operation boils down to one or more of these
-	CCXX_MEMBER_EXPORT(void) writeBinary(const uint8* data, const uint32 size) THROWS (Exception);
+	void writeBinary(const uint8* data, const uint32 size) THROWS (Exception);
 
 
 	// Read Operations
@@ -314,12 +323,12 @@ public:
  	/**
 	 * reads a BaseObject into a reference overwriting the object.
 	 */
-	CCXX_MEMBER_EXPORT(void) read(BaseObject &object) THROWS (Exception);
+	void read(BaseObject &object) THROWS (Exception);
 
   /**
 	 * reads a BaseObject into a pointer allocating memory for the object if necessary.
 	 */
-	CCXX_MEMBER_EXPORT(void) read(BaseObject *&object) THROWS (Exception);
+	void read(BaseObject *&object) THROWS (Exception);
 
 	// reads supported primitive types
   // shortcut, to make the following more readable
@@ -330,16 +339,18 @@ public:
 	void read(uint16& i) THROWS (Exception) { CCXX_ENGINEREAD_REF(i); }
 	void read(int32& i)  THROWS (Exception) { CCXX_ENGINEREAD_REF(i); }
 	void read(uint32& i) THROWS (Exception) { CCXX_ENGINEREAD_REF(i); }
+#ifdef	HAVE_64_BITS
 	void read(int64& i)  THROWS (Exception) { CCXX_ENGINEREAD_REF(i); }
 	void read(uint64& i) THROWS (Exception) { CCXX_ENGINEREAD_REF(i); }
+#endif
 	void read(float& i)  THROWS (Exception) { CCXX_ENGINEREAD_REF(i); }
 	void read(double& i) THROWS (Exception) { CCXX_ENGINEREAD_REF(i); }
 #undef CCXX_ENGINEREAD_REF
 
-	CCXX_MEMBER_EXPORT(void) read(std::string& str) THROWS (Exception);
+	void read(String& str) THROWS (Exception);
 
 	// Every read operation boild down to one or more of these
-	CCXX_MEMBER_EXPORT(void) readBinary(uint8* data, uint32 size) THROWS (Exception);
+	void readBinary(uint8* data, uint32 size) THROWS (Exception);
 
 private:
   /**
@@ -351,7 +362,7 @@ private:
   /**
    * reads in a class name, and caches it into the ClassMap.
    */
-  const std::string readClass();
+  const String readClass();
 
 
 	/**
@@ -369,8 +380,8 @@ private:
 	 */
 	typedef std::vector<BaseObject*>           ArchiveVector;
 	typedef std::map<BaseObject const*, int32> ArchiveMap;
-	typedef std::vector<std::string>                ClassVector;
-	typedef std::map<std::string, int32>            ClassMap;
+	typedef std::vector<String>                ClassVector;
+	typedef std::map<String, int32>            ClassMap;
 	
 	ArchiveVector myArchiveVector;
 	ArchiveMap myArchiveMap;
@@ -388,73 +399,75 @@ private:
 
 // Standard >> and << stream operators for BaseObject
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, BaseObject &ob)      THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, BaseObject &ob)      THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, BaseObject *&ob)      THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, BaseObject *&ob)      THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, BaseObject const &ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, BaseObject const &ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, BaseObject const *ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, BaseObject const *ob) THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, int8& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, int8& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, int8 ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, int8 ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, uint8& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, uint8& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, uint8 ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, uint8 ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, int16& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, int16& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, int16 ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, int16 ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, uint16& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, uint16& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, uint16 ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, uint16 ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, int32& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, int32& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, int32 ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, int32 ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, uint32& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, uint32& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, uint32 ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, uint32 ob)  THROWS (Engine::Exception);
+
+#ifdef	HAVE_64_BITS
+/** @relates Engine */
+__EXPORT Engine& operator >>( Engine& ar, int64& ob) THROWS (Engine::Exception);
+/** @relates Engine */
+__EXPORT Engine& operator <<( Engine& ar, int64 ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, int64& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, uint64& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, int64 ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, uint64 ob)  THROWS (Engine::Exception);
+#endif
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, uint64& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, float& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, uint64 ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, float ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, float& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, double& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, float ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, double ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, double& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, String& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, double ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, String ob)  THROWS (Engine::Exception);
 
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, std::string& ob) THROWS (Engine::Exception);
+__EXPORT Engine& operator >>( Engine& ar, bool& ob) THROWS (Engine::Exception);
 /** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, std::string ob)  THROWS (Engine::Exception);
-
-/** @relates Engine */
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, bool& ob) THROWS (Engine::Exception);
-/** @relates Engine */
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, bool ob)  THROWS (Engine::Exception);
+__EXPORT Engine& operator <<( Engine& ar, bool ob)  THROWS (Engine::Exception);
 
 /**
  * The following are templated classes
@@ -469,7 +482,7 @@ template<class T>
 Engine& operator <<( Engine& ar, typename std::vector<T> const& ob) THROWS (Engine::Exception)
 {
 	ar << (uint32)ob.size();
-	for(uint i=0; i < ob.size(); ++i)
+	for(unsigned int i=0; i < ob.size(); ++i)
 		ar << ob[i];
 	return ar;
 }
@@ -561,7 +574,7 @@ Engine& operator >>( Engine& ar, typename std::map<Key,Value>& ob) THROWS (Engin
 }
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 #endif
