@@ -40,11 +40,10 @@
 
 #include <cc++/config.h>
 #include <cc++/export.h>
+#include <cc++/slog.h>
 #include <cc++/file.h>
 #include "private.h"
 
-using std::string;
-
 #ifdef	CCXX_NAMESPACES
 namespace ost {
 #endif
@@ -133,6 +132,12 @@ void DSO::loader(const char *filename, bool flag)
 	void (*init)(void);
 #endif
 
+	id = strrchr(filename, '/');
+	if(id)
+		++id;
+	else
+		id = filename;
+
 	next = prev = NULL;
 
 #if defined(WIN32)
@@ -191,19 +196,22 @@ void DSO::loader(const char *filename, bool flag)
 #else
 	if(!image)
 	{
+		err = dlerror();
 #endif
 
-		if(getException() == Thread::throwObject)
+// since generally failure to map or load a plugin is fatel in most
+// cases, we should generally log the error to syslog as well as notify
+// the upper level system of the failure through exception.
+
+		slog.error() << "dso: " << id << ": " << err << std::endl;
+
+#ifdef	CCXX_EXCEPTIONS
+		if(Thread::getException() == Thread::throwObject)
 			throw(this);
 #ifdef	COMMON_STD_EXCEPTION
-		else if(getException() == Thread::throwException)
-			throw(DSOException(string(filename) + 
-#if defined(WIN32) || defined(HAVE_MACH_DYLD) || defined(HAVE_SHL_LOAD)
-					   err));
-#else
-					   dlerror()));
-#endif //WIN32
-
+		else if(Thread::getException() == Thread::throwException)
+			throw(DSOException(String(id) + err));
+#endif
 #endif
 		return;
 	}
@@ -221,7 +229,28 @@ void DSO::loader(const char *filename, bool flag)
 	mutex.leaveMutex();
 }	
 
+DSO *DSO::getObject(const char *id)
+{
+	const char *chk = strrchr(id, '/');
+	DSO *dso;
 
+	if(chk)
+		++chk;
+	else
+		chk = id;
+
+	mutex.enterMutex();
+	dso = first;
+	while(dso)
+	{
+		if(!stricmp(dso->id, chk))
+			break;
+		dso = dso->next;
+	}
+	mutex.leaveMutex();
+	return dso;
+}
+	
 bool DSO::isValid(void)
 {
 #if	defined(WIN32)
@@ -263,19 +292,10 @@ void *DSO::operator[](const char *sym)
 #endif
 }
 
-char *DSO::getError(void)
-{
-#if	defined(HAVE_SHL_LOAD) || defined(HAVE_MACH_DYLD) || defined(WIN32)
-	return err;
-#else
-	return (char *)dlerror();
-#endif
-}
-
 #endif
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 /** EMACS **
