@@ -39,6 +39,11 @@
 // If you do not wish that, delete this exception notice.
 
 #include <cc++/config.h>
+#include <cc++/string.h>
+#include <cc++/exception.h>
+
+#if	!defined(_MSC_VER) || _MSC_VER >= 1300
+
 #include <cc++/export.h>
 #include <cc++/persist.h>
 #include "assert.h"
@@ -52,11 +57,14 @@ using namespace std;
 const uint32 MAX_BUFFER = 16384;
 #endif
 
-Engine::Exception::Exception(const std::string &reason):
+#ifdef	CCXX_EXCEPTIONS
+Engine::Exception::Exception(const String &reason):
 	PersistException(reason) 
 {
 }
 
+#endif
+
 /**
  * NullObject is a const uint32 which is the ID streamed to disk
  * if an attempt to stream a NULL Persistence::BaseObject or
@@ -84,7 +92,7 @@ Engine::Engine(std::iostream& stream, EngineMode mode) THROWS (PersistException)
 	  myZStream.avail_out = MAX_BUFFER;
 	  int err = inflateInit(&myZStream);
 	  if (err != Z_OK) {
-		  THROW (PersistException(string("zLib didn't initialise for inflating.")));
+		  THROW (PersistException(String("zLib didn't initialise for inflating.")));
 	  }
 	}
   else
@@ -95,7 +103,7 @@ Engine::Engine(std::iostream& stream, EngineMode mode) THROWS (PersistException)
 	  myZStream.avail_out = MAX_BUFFER;
 	  int err = deflateInit(&myZStream,9); // TODO: tweak compression level
 	  if (err != Z_OK) {
-		  THROW ( PersistException(string("zLib didn't initialise for deflating.")));
+		  THROW ( PersistException(String("zLib didn't initialise for deflating.")));
 	  }
 	}
 #endif
@@ -221,8 +229,8 @@ void Engine::readBinary(uint8* data, uint32 size) THROWS (Engine::Exception)
 		  // Maybe we can still inflate some
 		  inflate(&myZStream,0);
 		  if (myZStream.avail_out == MAX_BUFFER)
-//			  THROW (PersistException(string("Oh dear - ran out of input")));
-			  THROW (Exception(string("Oh dear - ran out of input")));
+//			  THROW (PersistException(String("Oh dear - ran out of input")));
+			  THROW (Exception(String("Oh dear - ran out of input")));
 		}
 	}
 #endif
@@ -251,13 +259,13 @@ void Engine::write(const BaseObject *object) THROWS (Engine::Exception)
   if (itor == myArchiveMap.end())
 	{
 	  // Unfortunately we need to serialise it - here we go ....
-	  uint32 id = myArchiveMap.size();
+	  uint32 id = (uint32)myArchiveMap.size();
 	  myArchiveMap[object] = id; // bumps id automatically for next one
 	  write(id);
 	  ClassMap::const_iterator classItor = myClassMap.find(object->getPersistenceID());
 	  if (classItor == myClassMap.end())
 		{
-		  uint32 classId = myClassMap.size();
+		  uint32 classId = (uint32)myClassMap.size();
 		  myClassMap[object->getPersistenceID()] = classId;
 		  write(classId);
 		  write(object->getPersistenceID());
@@ -266,7 +274,7 @@ void Engine::write(const BaseObject *object) THROWS (Engine::Exception)
 		{
 		  write(classItor->second);
 		}
-	  std::string majik;
+	  String majik;
 	  majik = "OBST";
 	  write(majik);
 	  object->write(*this);
@@ -328,7 +336,7 @@ void Engine::read(BaseObject *&object) THROWS (Engine::Exception)
 	}
 
   // Okay - read the identifier for the class in...
-  std::string className = readClass();
+  String className = readClass();
   
   // is the pointer already initialized? if so then no need to reallocate
   if (object != NULL)
@@ -345,7 +353,7 @@ void Engine::read(BaseObject *&object) THROWS (Engine::Exception)
     readObject(object);
 	}
   else
-	  THROW (Exception((std::string("Unable to instantiate object of class ")+className).c_str()));
+	  THROW (Exception((String("Unable to instantiate object of class ")+className).c_str()));
 }
 
 /*
@@ -355,23 +363,23 @@ void Engine::readObject(BaseObject* object)
 {
 	// Okay then - we can make this object
 	myArchiveVector.push_back(object);
-	std::string majik;
+	String majik;
 	read(majik);
-	assert(majik == string("OBST"));
+	assert(majik == String("OBST"));
 	object->read(*this);
 	read(majik);
-	assert(majik == string("OBEN"));
+	assert(majik == String("OBEN"));
 }
 
 /*
  * reads the class information in
  */
-const string Engine::readClass()
+const String Engine::readClass()
 {
   // Okay - read the identifier for the class in...
   uint32 classId = 0;
   read(classId);
-  std::string className;
+  String className;
   if (classId < myClassVector.size())
 	{
 	  className = myClassVector[classId];
@@ -391,15 +399,15 @@ const string Engine::readClass()
  * note, does not (yet?) throw an exception, but interface
  * prepared ..
  */
-void Engine::write(const std::string& str) THROWS (Engine::Exception)
+void Engine::write(const String& str) THROWS (Engine::Exception)
 {
   assert(myOperationalMode == modeWrite);
-  uint32 len = str.length();
+  uint32 len = (uint32)str.length();
   write(len);
   writeBinary((uint8*)str.c_str(),len);
 }
 
-void Engine::read(std::string& str) THROWS (Engine::Exception)
+void Engine::read(String& str) THROWS (Engine::Exception)
 {
   assert(myOperationalMode == modeRead);
   uint32 len = 0;
@@ -437,11 +445,13 @@ CCXX_EXPORT(Engine&) operator <<( Engine& ar, int32 ob)  THROWS (Engine::Excepti
 CCXX_EXPORT(Engine&) operator >>( Engine& ar, uint32& ob) THROWS (Engine::Exception) {CCXX_RE (ar,ob);}
 CCXX_EXPORT(Engine&) operator <<( Engine& ar, uint32 ob)  THROWS (Engine::Exception) {CCXX_WE (ar,ob);}
 
+#ifdef	HAVE_64_BITS
 CCXX_EXPORT(Engine&) operator >>( Engine& ar, int64& ob) THROWS (Engine::Exception) {CCXX_RE (ar,ob);}
 CCXX_EXPORT(Engine&) operator <<( Engine& ar, int64 ob)  THROWS (Engine::Exception) {CCXX_WE (ar,ob);}
 
 CCXX_EXPORT(Engine&) operator >>( Engine& ar, uint64& ob) THROWS (Engine::Exception) {CCXX_RE (ar,ob);}
 CCXX_EXPORT(Engine&) operator <<( Engine& ar, uint64 ob)  THROWS (Engine::Exception) {CCXX_WE (ar,ob);}
+#endif
 
 CCXX_EXPORT(Engine&) operator >>( Engine& ar, float& ob) THROWS (Engine::Exception) {CCXX_RE (ar,ob);}
 CCXX_EXPORT(Engine&) operator <<( Engine& ar, float ob)  THROWS (Engine::Exception) {CCXX_WE (ar,ob);}
@@ -449,8 +459,8 @@ CCXX_EXPORT(Engine&) operator <<( Engine& ar, float ob)  THROWS (Engine::Excepti
 CCXX_EXPORT(Engine&) operator >>( Engine& ar, double& ob) THROWS (Engine::Exception) {CCXX_RE (ar,ob);}
 CCXX_EXPORT(Engine&) operator <<( Engine& ar, double ob)  THROWS (Engine::Exception) {CCXX_WE (ar,ob);}
 
-CCXX_EXPORT(Engine&) operator >>( Engine& ar, std::string& ob) THROWS (Engine::Exception) {CCXX_RE (ar,ob);}
-CCXX_EXPORT(Engine&) operator <<( Engine& ar, std::string ob)  THROWS (Engine::Exception) {CCXX_WE (ar,ob);}
+CCXX_EXPORT(Engine&) operator >>( Engine& ar, String& ob) THROWS (Engine::Exception) {CCXX_RE (ar,ob);}
+CCXX_EXPORT(Engine&) operator <<( Engine& ar, String ob)  THROWS (Engine::Exception) {CCXX_WE (ar,ob);}
 
 CCXX_EXPORT(Engine&) operator >>( Engine& ar, bool& ob) THROWS (Engine::Exception) {
 	uint32 a; ar.read(a); ob=a==1;return ar;
@@ -463,7 +473,9 @@ CCXX_EXPORT(Engine&) operator <<( Engine& ar, bool ob) THROWS (Engine::Exception
 #undef CCXX_WE
 
 #ifdef	CCXX_NAMESPACES
-};
+}
+#endif
+
 #endif
 
 /** EMACS **
