@@ -55,22 +55,29 @@
 
 #include <cc++/config.h>
 #include <cc++/export.h>
+#include <cc++/thread.h>
 #include <cc++/file.h>
 #include "private.h"
 
+#ifdef	__BORLANDC__
+#include <stdio.h>
+#else
 #include <cstdio>
+#endif
 #include <sys/stat.h>
 #include <cerrno>
 
-using std::string;
-
 #ifndef WIN32
 
+#ifdef	HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+
 #ifdef	HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
 
-#ifdef	HAVE_SYS_LOCKF_H
+#ifdef  HAVE_SYS_LOCKF_H
 #include <sys/lockf.h>
 #endif
 
@@ -79,8 +86,8 @@ using std::string;
 #undef	LOCK_SH
 #endif
 
-#ifdef	MACOSX
-#define	MISSING_LOCKF
+#ifdef  MACOSX
+#define MISSING_LOCKF   
 #endif
 
 #ifndef	F_LOCK
@@ -96,37 +103,6 @@ enum
 
 #endif
 
-#ifdef	MISSING_LOCKF
-
-static	int lockf(int fd, int cmd, long len)
-{
-	struct	flock lck;
-
-	lck.l_start = 0l;
-	lck.l_whence = SEEK_CUR;
-	lck.l_len = len;
-
-	switch(cmd)
-	{
-	case F_ULOCK:
-		lck.l_type = F_UNLCK;
-		return fcntl(fd, F_SETLK, &lck);
-	case F_LOCK:
-		lck.l_type = F_WRLCK;
-		return fcntl(fd, F_SETLKW, &lck);
-	case F_TLOCK:
-		lck.l_type = F_WRLCK;
-		return fcntl(fd, F_SETLK, &lck);
-	case F_TEST:
-		lck.l_type = F_WRLCK;
-		fcntl(fd, F_GETLK, &lck);
-		if(lck.l_type == F_UNLCK)
-			return 0;
-		return -1;
-	}
-}
-#endif // ndef F_LOCK
-
 #endif // ndef WIN32
 		
 
@@ -152,7 +128,7 @@ static const char *clearfifo(const char *pathname, int mode)
 namespace ost {
 #endif
 
-RandomFile::RandomFile() {
+RandomFile::RandomFile(const char *name) : Mutex(name) {
 #ifdef WIN32
 	fd = INVALID_HANDLE_VALUE;
 	// immediate is not defined on Win32 
@@ -162,10 +138,10 @@ RandomFile::RandomFile() {
 #endif
 	flags.thrown = flags.initial = flags.temp = false;
 	flags.count = 0;
+	pathname = NULL;
 }
 
-RandomFile::RandomFile(const RandomFile &rf) : Mutex()
-{
+RandomFile::RandomFile(const RandomFile &rf) : Mutex() {
 	// first, `dup'-licate the file descriptor/handle
 #ifdef WIN32
 	HANDLE pidHandle = GetCurrentProcess();
@@ -193,10 +169,7 @@ RandomFile::RandomFile(const RandomFile &rf) : Mutex()
 	flags.count = 0;
 
 	if(rf.pathname)
-	{
-		pathname = new char[strlen(rf.pathname) + 1];
-		strcpy(pathname, rf.pathname);
-	}
+		pathname = newString(rf.pathname);		
 	else
 		pathname = NULL;
 }
@@ -207,7 +180,7 @@ void RandomFile::final(void)
 	if(fd != INVALID_HANDLE_VALUE)
 	{
 		CloseHandle(fd);
-		if(flags.temp)
+		if(flags.temp && pathname)
 			DeleteFile(pathname);
 	}
 
@@ -215,14 +188,14 @@ void RandomFile::final(void)
 	if(fd > -1)
 	{
 		close(fd);
-		if(flags.temp)
+		if(flags.temp && pathname)
 			remove(pathname);
 	}
 #endif
 
 	if(pathname)
 	{
-		delete[] pathname;
+		delString(pathname);
 		pathname = NULL;
 	}
 
@@ -242,15 +215,17 @@ RandomFile::Error RandomFile::error(Error id, char *str)
 	if(!flags.thrown)
 	{
 		flags.thrown = true;
-		if(getException() == Thread::throwObject)
+#ifdef	CCXX_EXCEPTIONS
+		if(Thread::getException() == Thread::throwObject)
 			throw(this);
 #ifdef	COMMON_STD_EXCEPTION
-		else if(getException() == Thread::throwException)
+		else if(Thread::getException() == Thread::throwException)
 		{
 			if(!str)
 				str = "";
 			throw FileException(str);
 		}
+#endif
 #endif
 	}
 	return id;
@@ -285,7 +260,9 @@ bool RandomFile::initial(void)
 		if(pathname)
 			DeleteFile(pathname);
 		fd = INVALID_HANDLE_VALUE;
+		leaveMutex();
 		error(errInitFailed);
+		return false;
 	}
 
 #else
@@ -296,11 +273,11 @@ bool RandomFile::initial(void)
 		fd = -1;
 		if(pathname)
 			remove(pathname);
-		// FIXME: exception safe, mutex ??
+		leaveMutex();
 		error(errInitFailed);
+		return false;
 	}
-	else
-		fchmod(fd, mode);
+	fchmod(fd, mode);
 #endif
 	
 	leaveMutex();
@@ -333,6 +310,7 @@ RandomFile::Error RandomFile::setCompletion(Complete mode)
 	case completionDelayed:
 		flag |= O_NDELAY;
 
+	//completionDeferred: ? (hen)
 	case completionDeferred:
 		break;
 	}
@@ -377,7 +355,7 @@ bool RandomFile::operator!(void)
 #endif
 }
 
-ThreadFile::ThreadFile(const char *path) : RandomFile()
+ThreadFile::ThreadFile(const char *path) : RandomFile(path)
 {
 	first = NULL;
 	open(path);
@@ -407,8 +385,9 @@ ThreadFile::Error ThreadFile::open(const char *path)
 
 	if(path != pathname)
 	{
-		pathname = new char[strlen(path) + 1];
-		strcpy(pathname, path);
+		if(pathname)
+			delString(pathname);
+		pathname = newString(path);
 	}
 
 	flags.initial = false;
@@ -468,6 +447,7 @@ ThreadFile::Error ThreadFile::fetch(caddr_t address, ccxx_size_t len, off_t pos)
 {
 	fcb_t *fcb = getFCB();
 #ifdef WIN32
+	Thread::Cancel save;
 	if(fd == INVALID_HANDLE_VALUE)
 #else
 	if(fd < 0)
@@ -486,13 +466,16 @@ ThreadFile::Error ThreadFile::fetch(caddr_t address, ccxx_size_t len, off_t pos)
 
 #ifdef WIN32
 	enterMutex();
+	save = Thread::enterCancel();
 	SetFilePointer(fd, fcb->pos, NULL, FILE_BEGIN);
 	DWORD count;
 	if(!ReadFile(fd, fcb->address, fcb->len, &count, NULL))
 	{
+		Thread::exitCancel(save);
 		leaveMutex();
 		return errReadFailure;
 	}
+	Thread::exitCancel(save);
 	leaveMutex();
 	if(count < fcb->len)
 		return errReadIncomplete;
@@ -547,13 +530,16 @@ ThreadFile::Error ThreadFile::update(caddr_t address, ccxx_size_t len, off_t pos
 
 #ifdef WIN32
 	enterMutex();
+	Thread::Cancel save = Thread::enterCancel();
 	SetFilePointer(fd, fcb->pos, NULL, FILE_BEGIN);
 	DWORD count;
 	if(!WriteFile(fd, fcb->address, fcb->len, &count, NULL))
 	{
+		Thread::exitCancel(save);
 		leaveMutex();
 		return errWriteFailure;
 	}
+	Thread::exitCancel(save);
 	leaveMutex();
 	if(count < fcb->len)
 		return errWriteIncomplete;
@@ -605,13 +591,16 @@ ThreadFile::Error ThreadFile::append(caddr_t address, ccxx_size_t len)
 	enterMutex();
 
 #ifdef WIN32
+	Thread::Cancel save = Thread::enterCancel();
 	fcb->pos = SetFilePointer(fd, 0l, NULL, FILE_END);
 	DWORD count;
 	if(!WriteFile(fd, fcb->address, fcb->len, &count, NULL))
 	{
+		Thread::exitCancel(save);
 		leaveMutex();
 		return errWriteFailure;
 	}
+	Thread::exitCancel(save);
 	leaveMutex();
 	if(count < fcb->len)
 		return errWriteIncomplete;
@@ -682,7 +671,7 @@ bool ThreadFile::operator--(void)
 }
 
 SharedFile::SharedFile(const char *path) :
-RandomFile()
+RandomFile(path)
 {
 	fcb.address = NULL;
 	fcb.len = 0;
@@ -711,8 +700,9 @@ SharedFile::Error SharedFile::open(const char *path)
 
 	if(path != pathname)
 	{
-		pathname = new char[strlen(path) + 1];
-		strcpy(pathname, path);
+		if(pathname)
+			delString(pathname);
+		pathname = newString(path);
 	}
 
 	flags.initial = false;
@@ -772,6 +762,7 @@ SharedFile::Error SharedFile::fetch(caddr_t address, ccxx_size_t len, off_t pos)
 		fcb.pos = pos;
 
 #ifdef WIN32
+	Thread::Cancel save = Thread::enterCancel();
 	OVERLAPPED over;
 	SetFilePointer(fd, fcb.pos, NULL, FILE_BEGIN);
 	over.hEvent = 0;
@@ -781,9 +772,11 @@ SharedFile::Error SharedFile::fetch(caddr_t address, ccxx_size_t len, off_t pos)
 	DWORD count;
 	if(!ReadFile(fd, fcb.address, fcb.len, &count, NULL))
 	{
+		Thread::exitCancel(save);
 		leaveMutex();
 		return errReadFailure;
 	}
+	Thread::exitCancel(save);
 	leaveMutex();
 	if(count < fcb.len)
 		return errReadIncomplete;
@@ -852,6 +845,7 @@ SharedFile::Error SharedFile::update(caddr_t address, ccxx_size_t len, off_t pos
 		fcb.pos = pos;
 
 #ifdef WIN32
+	Thread::Cancel save = Thread::enterCancel();
 	OVERLAPPED over;
 	SetFilePointer(fd, fcb.pos, NULL, FILE_BEGIN);
 	over.hEvent = 0;
@@ -862,11 +856,13 @@ SharedFile::Error SharedFile::update(caddr_t address, ccxx_size_t len, off_t pos
 	{
 		SetFilePointer(fd, fcb.pos, NULL, FILE_CURRENT);
 		UnlockFileEx(fd, 0, len, 0, &over);
+		Thread::exitCancel(save);
 		leaveMutex();
 		return errWriteFailure;
 	}
 	SetFilePointer(fd, fcb.pos, NULL, FILE_CURRENT);
 	UnlockFileEx(fd, 0, len, 0, &over);
+	Thread::exitCancel(save);
 	leaveMutex();
 	if(count < fcb.len)
 		return errWriteIncomplete;
@@ -912,6 +908,7 @@ SharedFile::Error SharedFile::append(caddr_t address, ccxx_size_t len)
 		fcb.len = len;
 
 #ifdef WIN32
+	Thread::Cancel save = Thread::enterCancel();
 	fcb.pos = SetFilePointer(fd, 0l, NULL, FILE_END);
 	OVERLAPPED over;
 	over.hEvent = 0;
@@ -924,12 +921,15 @@ SharedFile::Error SharedFile::append(caddr_t address, ccxx_size_t len)
 	if(!WriteFile(fd, fcb.address, fcb.len, &count, NULL))
 	{
 		SetFilePointer(fd, eof, NULL, FILE_CURRENT);
+		Thread::exitCancel(save);
 		UnlockFileEx(fd, 0, 0x7fffffff, 0, &over);
+		Thread::exitCancel(save);
 		leaveMutex();
 		return errWriteFailure;
 	}
 	SetFilePointer(fd, eof, NULL, FILE_CURRENT);
 	UnlockFileEx(fd, 0, 0x7fffffff, 0, &over);
+	Thread::exitCancel(save);
 	leaveMutex();
 	if(count < fcb.len)
 		return errWriteIncomplete;
@@ -1002,9 +1002,152 @@ bool SharedFile::operator--(void)
 	return false;
 }
 
-#ifndef WIN32
+#ifdef WIN32
+
+static void makemapname(const char *source, char *target)
+{
+	unsigned count = 60;
+	while(*source && count--)
+	{
+		if(*source == '/' || *source == '\\')
+			*(target++) = '_';
+		else
+			*(target++) = toupper(*source);
+		++source;
+	}
+	*target = 0;
+}
+
 MappedFile::MappedFile(const char *fname, Access mode) :
-RandomFile()
+RandomFile(fname)
+{
+	DWORD share, page;
+	map = INVALID_HANDLE_VALUE;
+	fcb.address = NULL;
+
+	switch(mode)
+	{
+	case accessReadOnly:
+		share = FILE_SHARE_READ;
+		page = PAGE_READONLY;
+		prot = FILE_MAP_READ;
+		break;
+	case accessWriteOnly:
+		share = FILE_SHARE_WRITE;
+		page = PAGE_WRITECOPY;
+		prot = FILE_MAP_COPY;
+		break;
+	case accessReadWrite:
+		share = FILE_SHARE_READ|FILE_SHARE_WRITE;
+		page = PAGE_READWRITE;
+		prot = FILE_MAP_WRITE;
+	}
+	fd = CreateFile(pathname, mode, share, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, NULL);
+	if(fd == INVALID_HANDLE_VALUE)
+	{
+		error(errOpenFailed);
+		return;
+	}
+	makemapname(fname, mapname);
+	map = CreateFileMapping(fd, NULL, page, 0, 0, mapname);
+	if(!map)
+		error(errMapFailed);
+}
+
+MappedFile::MappedFile(const char *fname, pos_t pos, size_t len, Access mode) :
+RandomFile(fname)
+{
+	DWORD share, page;
+	map = INVALID_HANDLE_VALUE;
+	fcb.address = NULL;
+
+	switch(mode)
+	{
+	case accessReadOnly:
+		share = FILE_SHARE_READ;
+		page = PAGE_READONLY;
+		prot = FILE_MAP_READ;
+		break;
+	case accessWriteOnly:
+		share = FILE_SHARE_WRITE;
+		page = PAGE_WRITECOPY;
+		prot = FILE_MAP_COPY;
+		break;
+	case accessReadWrite:
+		share = FILE_SHARE_READ|FILE_SHARE_WRITE;
+		page = PAGE_READWRITE;
+		prot = FILE_MAP_WRITE;
+	}
+	fd = CreateFile(pathname, mode, share, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, NULL);
+	if(fd == INVALID_HANDLE_VALUE)
+	{
+		error(errOpenFailed);
+		return;
+	}
+	makemapname(fname, mapname);
+	map = CreateFileMapping(fd, NULL, page, 0, 0, mapname);
+	if(!map)
+	{
+		error(errMapFailed);
+		return;
+	}
+	fcb.address = MapViewOfFile(map, prot, 0, pos, len);
+	fcb.len = (ccxx_size_t)len;
+	fcb.pos = pos;
+	if(!fcb.address)
+		error(errMapFailed);
+}
+
+MappedFile::~MappedFile()
+{
+	if(fcb.address)
+		UnmapViewOfFile(fcb.address);
+
+	if(map != INVALID_HANDLE_VALUE)
+		CloseHandle(map);
+
+	final();
+}
+
+void MappedFile::sync(void)
+{
+}
+
+void MappedFile::sync(caddr_t address, size_t len)
+{
+}
+
+void MappedFile::release(caddr_t address, size_t len)
+{
+	if(fcb.address)
+		UnmapViewOfFile(fcb.address);
+	fcb.address = NULL;
+}
+
+caddr_t MappedFile::fetch(off_t pos, size_t len)
+{
+	if(fcb.address)
+		UnmapViewOfFile(fcb.address);
+
+	fcb.address = MapViewOfFile(map, prot, 0, pos, len);
+	fcb.len = (ccxx_size_t)len;
+	fcb.pos = pos;
+	if(!fcb.address)
+		error(errMapFailed);
+
+	return fcb.address;
+}
+void MappedFile::update(size_t offset, size_t len)
+{
+}
+
+void MappedFile::update(caddr_t address, size_t len)
+{
+}
+
+#else
+MappedFile::MappedFile(const char *fname, Access mode) :
+RandomFile(fname)
 {
 	fd = open(fname, (int)mode);
 	if(fd < 0 && mode != accessReadOnly)
@@ -1031,7 +1174,7 @@ RandomFile()
 }
 
 MappedFile::MappedFile(const char *fname, pos_t pos, size_t len, Access mode) :
-RandomFile()
+RandomFile(fname)
 {
 	fd = open(fname, (int)mode);
 	if(fd < 0)
@@ -1062,7 +1205,7 @@ RandomFile()
 	{
 		close(fd);
 		fd = -1;
-		error(errMmapFailed);
+		error(errMapFailed);
 	}
 }
 
@@ -1071,6 +1214,11 @@ MappedFile::~MappedFile()
 	final();
 }
 
+void MappedFile::sync(void)
+{
+	msync(fcb.address, fcb.len, MS_SYNC);
+}
+
 void MappedFile::release(caddr_t address, size_t len)
 {
 	enterMutex();
@@ -1236,8 +1384,363 @@ bool canModify(const char *path)
 #endif
 }
 
+#ifdef	WIN32
+
+const char *File::getExtension(const char *path)
+{
+        const char *cp = strrchr(path, '\\');
+        if(!cp)
+                cp = strrchr(path, '/');
+        if(!cp)
+                cp = strrchr(path, ':');
+
+
+	if(cp)
+		++cp;
+	else
+		cp = path;
+
+	if(*cp == '.')
+		return "";
+
+	cp = strrchr(cp, '.');
+
+	if(!cp)
+		cp = "";
+
+	return cp;
+}
+
+const char *File::getFilename(const char *path)
+{
+	const char *cp = strrchr(path, '\\');
+	if(cp)
+		return ++cp;
+
+	cp = strrchr(path, '/');
+	if(cp)
+		return ++cp;
+
+	cp = strrchr(path, ':');
+	if(cp)
+		++cp;
+
+	return path;
+}
+
+char *File::getFilename(const char *path, char *buffer, size_t size)
+{
+	const char *cp = strrchr(path, '\\');
+	if(!cp)
+		cp = strrchr(path, '/');
+	if(!cp)
+		cp = strrchr(path, ':');
+
+	if(cp)
+		snprintf(buffer, size, "%s", ++cp);
+	else
+		snprintf(buffer, size, "%s", path);
+
+	return buffer;
+}
+
+char *File::getDirname(const char *path, char *buffer, size_t size)
+{
+	size_t len;
+        const char *cp = strrchr(path, '\\');
+
+	snprintf(buffer, size, "%s", path);
+
+        if(!cp)
+                cp = strrchr(path, '/');
+        if(!cp)
+                cp = strrchr(path, ':');
+
+	if(!cp)
+		return buffer;
+
+	if(cp)
+		len = cp - path;
+
+	if(len >= size)
+		len = size - 1;
+	buffer[len] = 0;
+	return buffer;
+}
+
+#else
+
+const char *File::getExtension(const char *path)
+{
+	const char *cp = strrchr(path, '/');
+	if(cp)
+		++cp;
+	else
+		cp = path;
+
+	if(*cp == '.')
+		return "";
+
+	cp = strrchr(cp, '.');
+	if(cp)
+		return cp;
+	return "";
+}
+
+char *File::getDirname(const char *path, char *buffer, size_t size)
+{
+        unsigned len;
+        const char *cp = strrchr(path, '/');
+
+        snprintf(buffer, size, "%s", path);
+
+        if(!cp)
+                return buffer;
+
+        if(cp)
+                len = cp - path;
+
+        if(len >= size)
+                len = size - 1;
+        buffer[len] = 0;
+        return buffer;
+}
+
+const char *File::getFilename(const char *path)
+{
+	const char *cp = strrchr(path, '/');
+
+	if(cp)
+		return ++cp;
+
+	return path;
+}
+
+char *File::getFilename(const char *path, char *buffer, size_t size)
+{
+	const char *cp = strrchr(path, '/');
+
+	if(cp)
+		snprintf(buffer, size, "%s", ++cp);
+	else
+		snprintf(buffer, size, "%s", path);
+
+	return buffer;
+}
+
+#endif
+
+#ifdef	HAVE_REALPATH
+
+char *File::getRealpath(const char *path, char *buffer, size_t size)
+{
+	char temp[PATH_MAX];
+	setString(buffer, size, ".");
+	if(!realpath(path, temp))
+		return NULL;
+	if(strlen(temp) >= size)
+		return NULL;
+	setString(buffer, size, temp);
+	return buffer;
+}
+
+#else
+
+#ifdef  WIN323
+static char *getFile(char *path)
+{
+        char *cp = strchr(path, '\\');
+        if(!cp)
+                cp = strchr(path, '/');
+        if(!cp)
+                cp = strchr(path, ':');
+        return cp;
+}
+#else
+static char *getFile(char *path)
+{
+	return strchr(path, '/');
+}
+#endif
+
+char *File::getRealpath(const char *path, char *buffer, size_t size)
+{
+	if(size > PATH_MAX)
+		size = PATH_MAX;
+
+	unsigned symlinks = 0;
+#if !defined(DYNAMCIC_LOCAL_ARRAYS)
+	char left[PATH_MAX];
+#else
+	char left[size];
+#endif
+	size_t left_len, buffer_len;
+
+#ifdef	WIN32
+	if(path[1] == ':')
+#else
+	if(path[0] == '/')
+#endif
+	{
+		buffer[0] = '/';
+		buffer[1] = 0;
+		if(!path[1])
+			return buffer;
+
+		buffer_len = 1;
+		snprintf(left, size, "%s", path + 1);
+		left_len = strlen(left);
+  	}
+	else
+	{
+		if(!Dir::getPrefix(buffer, size))
+		{
+			snprintf(buffer, size, "%s", ".");
+			return NULL;
+		}
+		buffer_len = strlen(buffer);
+		snprintf(left, size, "%s", path);
+		left_len = strlen(left);
+	}
+
+	if(left_len >= size || buffer_len >= size)
+		return NULL;
+
+	while(left_len > 0)
+	{
+#ifdef	HAVE_LSTAT
+		struct stat ino;
+#endif
+#if !defined(DYNAMIC_LOCAL_ARRAYS)
+		char next_token[PATH_MAX];
+#else
+		char next_token[size];
+#endif
+		char *p;
+		const char *s = (p = getFile(left)) ? p : left + left_len;
+
+		memmove(next_token, left, s - left);
+		left_len -= s - left;
+		if(p != NULL)
+			memmove(left, s + 1, left_len + 1);
+
+		next_token[s - left] = 0;
+		if(buffer[buffer_len - 1] != '/')
+		{
+			if(buffer_len +1 >= size)
+				return NULL;
+
+			buffer[buffer_len++] = '/';
+			buffer[buffer_len] = 0;
+		}
+		if(!next_token[0])
+			continue;
+		else if(!strcmp(next_token, "."))
+			continue;
+		else if(!strcmp(next_token, ".."))
+		{
+			if(buffer_len > 1)
+			{
+				char *q;
+				buffer[buffer_len - 1] = 0;
+#ifdef	WIN32
+				q = strrchr(buffer, '\\');
+				if(!q)
+					q = strrchr(buffer, '/');
+				if(!q)
+					q = strchr(buffer, ':');
+#else
+				q = strrchr(buffer, '/');
+#endif
+				*q = 0;
+				buffer_len = q - buffer;
+			}
+			continue;
+		}
+		snprintf(next_token, size, "%s", buffer);
+		buffer_len = strlen(buffer);
+		if(buffer_len >= size)
+			return NULL;
+
+#ifndef	HAVE_LSTAT
+		if(!isFile(buffer) && !isDir(buffer))
+			return buffer;
+
+		continue;
+#else
+		if(lstat(buffer, &ino) < 0)
+		{
+			if(errno == ENOENT && !p)
+				return buffer;
+
+			return NULL;
+		}
+
+		if((ino.st_mode & S_IFLNK) == S_IFLNK)
+		{
+                	char symlink[size];
+                        int slen;
+
+                        if (symlinks++ > MAXSYMLINKS)
+				return NULL;
+
+                        slen = readlink(buffer, symlink, size);
+
+                        if (slen < 0)
+                                return NULL;
+                        symlink[slen] = 0;
+
+                        if (symlink[0] == '/') 
+			{
+                                buffer[1] = 0;
+                                buffer_len = 1;
+                        } else if (buffer_len > 1) 
+			{
+                                char *q;
+
+				buffer[buffer_len - 1] = 0;
+                                q = strrchr(buffer, '/');
+                                *q = 0;
+                                buffer_len = q - buffer;
+                        }
+                        if (symlink[slen - 1] != '/' && p) 
+			{
+                                if (slen >= size)
+					return NULL;
+
+                                symlink[slen] = '/';
+                                symlink[slen + 1] = 0;
+                        }
+                        if(p)
+			{
+				snprintf(symlink, size, "%s", left);
+				left_len = strlen(symlink);
+			}
+			if(left_len >= size)
+				return NULL;
+			snprintf(left, size, "%s", symlink);
+			left_len = strlen(left);
+		}
+#endif
+
+	}
+
+#ifdef	WIN32
+        if(buffer_len > 1 && buffer[buffer_len - 1] == '\\')
+                buffer[buffer_len - 1] = 0;
+        else if(buffer_len > 1 && buffer[buffer_len - 1] == '/')
+                buffer[buffer_len - 1] = 0;
+#else
+	if(buffer_len > 1 && buffer[buffer_len - 1] == '/')
+		buffer[buffer_len - 1] = 0;
+#endif
+	return buffer;
+}
+
+#endif
+
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 /** EMACS **
