@@ -13,7 +13,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-// 
+//
 // As a special exception to the GNU General Public License, permission is 
 // granted for additional uses of the text contained in its release 
 // of Common C++.
@@ -40,14 +40,25 @@
 
 #include <cc++/config.h>
 #include <cc++/export.h>
+#include <cc++/thread.h>
 #include <cc++/process.h>
+#include <cc++/strchar.h>
 
 #include <cstdlib>
 #include <cstdio>
+#include <cerrno>
 
 #ifndef	WIN32
+#ifdef	HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+
+#ifdef	HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
+
+#include <pwd.h>
+#include <grp.h>
 
 #ifdef	SIGTSTP
 #include <sys/file.h>
@@ -68,15 +79,210 @@
 namespace ost {
 #endif
 
+static char *_pUser = NULL;
+static char *_pHome = NULL;
+
 #ifdef	WIN32
-int Process::spawn(const char *exename, const char **args)
+int Process::spawn(const char *exename, const char **args, bool wait)
 {
-	return ::spawnvp(P_WAIT, (char *)exename, (char **)args);
+	int mode = P_NOWAIT;
+
+	if(wait)
+			mode = P_WAIT;
+
+	return (int)::spawnvp(mode, (char *)exename, (char **)args);
 }
-#else
-int Process::spawn(const char *exename, const char **args)
+
+int Process::join(int pid)
 {
-	int status, pid;
+	int status, result;
+
+	if(pid == -1)
+		return pid;
+
+	result = (int)cwait(&status, pid, WAIT_CHILD);
+	if(status & 0x0)
+		return -1;
+	return result;
+}
+
+bool Process::cancel(int pid, int sig)
+{
+	HANDLE hPid = OpenProcess(PROCESS_ALL_ACCESS, TRUE, pid);
+	bool rtn = true;
+
+	if(!hPid)
+		return false;
+
+	if(!sig)
+		sig = 2;
+
+	if(!TerminateProcess(hPid, sig))
+		rtn = false;
+
+	CloseHandle(hPid);
+	return rtn;
+}
+
+#else
+
+#ifndef WEXITSTATUS
+#define WEXITSTATUS(status) ((unsigned)(status) >> 8)
+#endif
+
+#ifndef WIFEXITED
+#define WIFEXITED(status) (((status) & 255) == 0)
+#endif
+
+#ifndef WTERMSIG
+#define WTERMSIG(status) (((unsigned)(status)) & 0x7F)
+#endif
+
+#ifndef WIFSIGNALLED
+#define WIFSIGNALLED(status) (((status) & 255) != 0)
+#endif
+
+#ifndef WCOREDUMP
+#define WCOREDUMP(status) (((status) & 0x80) != 0)
+#endif
+
+static void lookup(void)
+{
+	struct passwd *pw = NULL;
+#ifdef	GETPWUID_R
+	struct passwd pwd;
+	char buffer[1024];
+
+	::getpwuid_r(geteuid(), &pwd, buffer, 1024, &pw);
+#else
+	pw = ::getpwuid(geteuid());
+#endif
+
+	if(_pHome)
+		delString(_pHome);
+	if(_pUser)
+		delString(_pUser);
+
+	_pUser = _pHome = NULL;
+
+	if(pw != NULL && pw->pw_dir != NULL)
+		_pHome = newString(pw->pw_dir);
+
+	if(pw != NULL && pw->pw_name != NULL)
+		_pUser = newString(pw->pw_name);
+
+	endpwent();
+}
+
+const char *Process::getUser(void)
+{
+	if(!_pUser)
+		lookup();
+
+	return (const char *)_pUser;
+}
+
+const char *Process::getHomeDir(void)
+{
+	if(!_pHome)
+		lookup();
+
+	return (const char *)_pHome;
+}
+
+bool Process::setUser(const char *id, bool grp)
+{
+        struct passwd *pw = NULL;
+#ifdef  GETPWNAM_R
+        struct passwd pwd;
+        char buffer[1024];
+
+        ::getpwnam_r(id, &pwd, buffer, 1024, &pw);
+#else
+        pw = ::getpwnam(id);
+#endif
+	if(!pw)
+		return false;
+
+	if(grp)
+		if(setgid(pw->pw_gid))
+			return false;
+
+	if(setuid(pw->pw_uid))
+		return false;
+
+	lookup();
+	return true;
+}
+
+bool Process::setGroup(const char *id)
+{
+	struct group *group = NULL;
+#ifdef	GETGRNAM_R
+	struct group grp;
+	char buffer[2048];
+
+	::getgrnam_r(id, &grp, buffer, 1024, &group);
+#else
+	group = ::getgrnam(id);
+#endif
+	if(!group)
+		return false;
+
+#ifdef	HAVE_SETEGID
+	setegid(group->gr_gid);
+#endif
+	if(setgid(group->gr_gid))
+		return false;
+
+	endgrent();
+	return true;
+}
+
+bool Process::cancel(int pid, int sig)
+{
+	if(!sig)
+		sig = SIGTERM;
+
+	if(pid < 1)
+		return false;
+
+	if(::kill(pid, sig))
+		return false;
+
+	return true;
+}
+
+int Process::join(int pid)
+{
+	int status;
+
+	if(pid < 1)
+		return -1;
+
+#ifdef  HAVE_WAITPID
+        waitpid(pid, &status, 0);
+#else
+#ifdef  HAVE_WAIT4
+        wait4(pid, &status, 0, NULL);
+#else
+	int result;
+        while((result = wait(&status)) != pid && result != -1)
+                ;
+#endif
+#endif
+
+        if(WIFEXITED(status))
+                return WEXITSTATUS(status);
+        else if(WIFSIGNALLED(status))
+                return -WTERMSIG(status);
+        else
+                return -1;
+}
+
+int Process::spawn(const char *exename, const char **args, bool wait)
+{
+	int pid;
 
 	pid = vfork();
 	if(pid == -1)
@@ -88,17 +294,10 @@ int Process::spawn(const char *exename, const char **args)
 		_exit(-1);
 	}
 
-#ifdef	__FreeBSD__
-	wait4(pid, &status, WUNTRACED, NULL);
-#else
-	waitpid(pid, &status, WUNTRACED);
-#endif
+	if(!wait)
+		return pid;
 
-#ifdef	WEXITSTATAUS
-	return WEXITSTATUS(status);
-#else
-	return status;
-#endif
+	return join(pid);
 }
 
 Process::Trap Process::setInterruptSignal(int signo, Trap func)
@@ -145,7 +344,12 @@ Process::Trap Process::setPosixSignal(int signo, Trap func)
 	return old_act.sa_handler;
 }
 
-void    Process::detach(void)
+void	Process::detach(void)
+{
+	attach("/dev/null");
+}
+
+void    Process::attach(const char *dev)
 {
         int pid;
         int fd;
@@ -153,6 +357,10 @@ void    Process::detach(void)
         if(getppid() == 1)
                 return;
 
+	::close(0);
+	::close(1);
+	::close(2);
+
 #ifdef  SIGTTOU
         ::signal(SIGTTOU, SIG_IGN);
 #endif
@@ -166,14 +374,14 @@ void    Process::detach(void)
 #endif
 
         if((pid = fork()) < 0)
-                throw(pid);
+                THROW(pid);
         else if(pid > 0)
                 exit(0);
 
 
 #if	defined(SIGTSTP) && defined(TIOCNOTTY)
         if(setpgid(0, getpid()) == -1)
-                throw(-1);
+                THROW(-1);
 
         if((fd = open(_PATH_TTY, O_RDWR)) >= 0)
         {
@@ -181,19 +389,32 @@ void    Process::detach(void)
                 close(fd);
         }
 #else
+
+#ifdef	HAVE_SETPGRP
         if(setpgrp() == -1)
-                throw(-1);
+                THROW(-1);
+#else
+	if(setpgid(0, getpid()) == -1)
+		THROW(-1);
+#endif
 
         ::signal(SIGHUP, SIG_IGN);
 
         if((pid = fork()) < 0)
-                throw(-1);
+                THROW(-1);
         else if(pid > 0)
                 exit(0);
+#endif
 
-#endif
+	if(dev && *dev)
+	{
+       		::open(dev, O_RDWR);
+        	::open(dev, O_RDWR);
+        	::open(dev, O_RDWR);
+	}
 }
-#endif
+
+#endif	// not win32
 
 void Process::setEnv(const char *name, const char *value, bool overwrite)
 {
@@ -207,7 +428,7 @@ void Process::setEnv(const char *name, const char *value, bool overwrite)
 		if(getenv(strbuf))
 			return;
 
-	::putenv(strbuf);
+	::putenv(strdup(strbuf));
 #endif
 }
 
@@ -217,7 +438,7 @@ const char *Process::getEnv(const char *name)
 }
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 /** EMACS **
